@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """utilities methods and classes for checkers
 
 Base id of standard checkers (used in msg and report ids):
@@ -41,7 +41,6 @@ messages nor reports. XXX not true, emit a 07 report !
 import sys
 import tokenize
 import warnings
-from os.path import dirname
 
 from astroid.utils import ASTWalker
 from logilab.common.configuration import OptionsProviderMixIn
@@ -92,8 +91,6 @@ class BaseChecker(OptionsProviderMixIn, ASTWalker):
         self.name = self.name.lower()
         OptionsProviderMixIn.__init__(self)
         self.linter = linter
-        # messages that are active for the current check
-        self.active_msgs = set()
 
     def add_message(self, msg_id, line=None, node=None, args=None):
         """add a message of a given type"""
