@@ -1,5 +1,7 @@
-# pylint: disable=R0903
+# pylint: disable=R0903,import-error
 """check use of super"""
+
+from unknown import Missing
 __revision__ = None
 
 class Aaaa:
@@ -29,3 +31,8 @@ class Py3kWrongSuper(Py3kAaaa):
     """new style"""
     def __init__(self):
         super(NewAaaa, self).__init__()
+
+class WrongNameRegression(Py3kAaaa):
+    """ test a regression with the message """
+    def __init__(self):
+        super(Missing, self).__init__()
