@@ -1,17 +1,17 @@
-# pylint: disable-msg=R0903,W0212,W0403,W0406
+# pylint: disable=R0903,W0212,W0403,W0406
 """test for call to __init__ from a non ancestor class
 """
 
 __revision__ = '$Id: func_w0233.py,v 1.2 2004-09-29 08:35:13 syt Exp $'
 
-class AAAA:
+class AAAA(object):
     """ancestor 1"""
 
     def __init__(self):
         print 'init', self
         BBBBMixin.__init__(self)
 
-class BBBBMixin:
+class BBBBMixin(object):
     """ancestor 2"""
 
     def __init__(self):
